
# Zope imports
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, Unauthorized

# CMF/Plone imports
from Products.CMFCore.CMFCorePermissions import SetOwnPassword, ManagePortal
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFCore import CMFCorePermissions
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFPlone.MembershipTool import MembershipTool as BaseTool
from Products.Archetypes.utils import OrderedDict

# Sibling imports

from Products.CMFMember.MemberPermissions import VIEW_PUBLIC_PERMISSION
from Products.CMFMember.utils import checkPermission

class MembershipTool( BaseTool ):
    meta_type='CMFMember Membership Tool'
    security = ClassSecurityInfo()
    plone_tool = 1

    def getPersonalPortrait(self, member_id=None, verifyPermission=0):
        """Returns the Portait for a member_id"""
        portal = getToolByName(self, 'portal_url').getPortalObject()
        default_portrait = getattr(portal, self.default_portrait)
        if not member_id:
            return default_portrait

        memberdata_tool = getToolByName(self, 'portal_memberdata')
        member = None
        try:
            member = memberdata_tool.get(member_id)
            if verifyPermission and not _checkPermission(VIEW_PUBLIC_PERMISSION, member):
                return default_portrait
            portrait = member.getPortrait()
            if portrait:
                return portrait
        except AttributeError:
            pass
        return default_portrait

    def changeMemberPortrait(self, portrait, member_id=None):
        """Override Plone's changeMemberPortrait method to use
        CMFMember's portrait management"""
        if not member_id:
            return
        memberdata_tool = getToolByName(self, 'portal_memberdata')
        member = memberdata_tool.get(member_id, None)
        if member:
            member.setPortrait(portrait)

    def searchForMembers( self, REQUEST=None, **kw ):
        """
        here for backwards compatibility; member searching is better
        accomplished using the member_catalog, which this ultimately
        delegates to
        """
        if REQUEST:
            param = getattr(REQUEST, 'form', REQUEST)
        else:
            param = kw

        # mapping from older lookup names to the indexes that exist
        # in the member_catalog
        key_map = {'name': 'getId',
                   'email': 'getEmail',
                   'roles': 'getFilteredRoles',
                   'last_login_time': 'getLastLoginTime',
                   'last_login_usage': 'getLastLoginTime_usage',
                   }
        for key, replacement in key_map.items():
            if not param.has_key(key):
                continue
            param[replacement] = param.pop(key)

            usage = "%s_usage" % key
            if not param.has_key("%s_usage" % key):
                continue
            ureplacement = "%s_usage" % replacement
            param[ureplacement] = param.pop(usage)
                
        memberdata_tool = getToolByName(self, 'portal_memberdata')
        return memberdata_tool.searchForMembers(REQUEST, **kw)

    security.declarePublic("createMemberArea")
    def createMemberArea(self, member_id=''):
        """ created hook for 'preCreateMemberArea' and 'postCreateMemberArea'
            if you provide methods of PythonScripts with these names, they will 
            be called at the begin and end of createMemberData
            
            if preCreateMemberArea returns 1 the normal createMemberArea will be
            called. otherwise it is skipped.
        """
        if not self.memberareaCreationFlag:
            try:
                checkPermission(self, ManagePortal, "createMemberArea")
            except Unauthorized:
                # XXX: Unfortunately we have to be silent here. Plone will barf otherwise.
                return
            
        # if preCreateMemberArea returns false,
        # no further creation of the member area takes place

        # do not create member_area for groups
        if hasattr(self, 'portal_groups') and member_id in self.portal_groups.listGroupIds():
            return
        
        pre = getattr(self, 'preCreateMemberArea', None)
        createarea = 1
        if pre:
            createarea = pre(member_id=member_id)
        if createarea:
            BaseTool.createMemberarea(self, member_id)
        post = getattr(self, 'postCreateMemberArea', None)
        if post:
            post(member_id=member_id)

    # XXX: Backwards compatibility for CMF API change, note the spelling
    security.declarePublic("createMemberarea")
    createMemberarea = createMemberArea

    security.declarePrivate('addMember')
    def addMember(self, id, password, roles, domains, properties=None):
        '''Adds a new member to the user folder.  Security checks will have
        already been performed.  Called by portal_registration.
        '''

        if hasattr(self, 'portal_memberdata'):
            memberdata_container = self.portal_memberdata
            member_type = 'Member'
            typeName = str(memberdata_container.getTypeName())
            if typeName:
                member_type = typeName
            memberdata_container.invokeFactory(member_type,id)
            member=getattr(memberdata_container.aq_explicit,id)
            member.edit(password=password,roles=roles,domains=domains,**(properties or {}))


    def getMemberById(self, id):
        ''' overload of CMFCore.MembershipTool.getMemberById, returns None if member doesnt exist '''
        if hasattr(self, 'portal_memberdata'):
            memberdata_container = self.portal_memberdata
            member=getattr(memberdata_container.aq_explicit,id,None)
            
        if member is not None:
            return member

        # if no member found, try to find the user
        u = self.acl_users.getUserById(id, None)
        if u is not None:
            # wrap the user and return a new member
            u = self.wrapUser(u)
        return u


    security.declareProtected(SetOwnPassword, 'setPassword')
    def setPassword(self, password, domains=None):
        '''Allows the authenticated member to set his/her own password.
        '''
        try:
            BaseTool.setPassword(self, password, domains)
            member = self.getAuthenticatedMember()
            member._setPassword(password)
        except (errorMsg, errorComment):
            raise errorMsg, errorComment

    security.declarePublic('areUsersAutomatic')
    def areUsersAutomatic(self):
        return false

InitializeClass(MembershipTool)
